#############################################################################
#
# MODULE:   	Grass Compilation
# AUTHOR(S):	Original author unknown - probably CERL
#		Markus Neteler - Germany/Italy - neteler@itc.it
#   	    	Justin Hickey - Thailand - jhickey@hpcc.nectec.or.th
#   	    	Huidae Cho - Korea - grass4u@gmail.com
#   	    	Eric G. Miller - egm2@jps.net
# PURPOSE:  	The source file for this Makefile is in src/CMD/head/head.in.
#		It is the top part of a file called make.rules which is used
#		for compiling all GRASS modules. This part of the file provides
#		make variables that are dependent on the results of the
#		configure script.
# COPYRIGHT:    (C) 2000 by the GRASS Development Team
#
#               This program is free software under the GNU General Public
#   	    	License (>=v2). Read the file COPYING that comes with GRASS
#   	    	for details.
#
#############################################################################

############################## Make Variables ###############################

CC                  = gcc
#FC                  = @F77@
CXX                 = c++
LEX                 = flex
YACC                = bison -y
PERL                = /usr/local/bin/perl
AR                  = ar
RANLIB              = ranlib
MKDIR               = mkdir -p
CHMOD               = chmod
INSTALL             = /usr/local/bin/ginstall -c 
INSTALL_DATA        = ${INSTALL} -m 644

ARCH                = i386-unknown-freebsd6.1
prefix              = /usr/local
exec_prefix         = ${prefix}
UNIX_BIN            = ${exec_prefix}/bin
INST_DIR            = /usr/local/grass-6.1.cvs
GRASSMANDIR         = ${prefix}/man

DL_LIBS             = @DL_LIBS@
DL_OBJS             = @DL_OBJS@
PLAT_OBJS           = 
CFLAGS_DEBUG        = -g
CFLAGS_OPTIMIZE     = -O
CFLAGS_WARNING      = -Wall -Wconversion -Wno-implicit-int
STRIPFLAG           = 
LDFLAGS_DEBUG       = 
LDFLAGS_OPTIMIZE    = 
CC_SEARCH_FLAGS     = -Wl,-rpath,${LIB_RUNTIME_DIR}
LD_SEARCH_FLAGS     = -rpath ${LIB_RUNTIME_DIR}
LD_LIBRARY_PATH_VAR = LD_LIBRARY_PATH
LIB_RUNTIME_DIR     = $(ARCH_LIBDIR)

#static libs:
STLIB_LD            = ${AR} cr
STLIB_PREFIX        = lib
STLIB_SUFFIX        = .a

#shared libs
SHLIB_PREFIX        = lib
SHLIB_LD            = ld -Bshareable -x
SHLIB_LD_EXTRAS     = 
SHLIB_LD_FLAGS      = 
SHLIB_LD_LIBS       = ${LIBS}
SHLIB_CFLAGS        = -fPIC
SHLIB_SUFFIX        = .so
EXE_SUFFIX          = 


# GRASS dirs
GRASS_HOME          = /usr/home/geni/usr/grass/grass6.transparency

DEFAULT_DATABASE    =
DEFAULT_LOCATION    =

CPPFLAGS            =   -I/usr/local/include
CFLAGS1             = -g -O2 
CXXFLAGS1           = -g -O2
INCLUDE_DIRS        =  -I/usr/local/include

COMPILE_FLAGS       = $(CPPFLAGS) $(CFLAGS1) $(CFLAGS_WARNING) $(INCLUDE_DIRS)
COMPILE_FLAGS_CXX   = $(CPPFLAGS) $(CXXFLAGS1) $(INCLUDE_DIRS)
LINK_FLAGS          =  -export-dynamic  -L/usr/local/lib

DLLIB               = 
XCFLAGS             =  -I/usr/X11R6/include
XLDFLAGS            = 
XMINC               = 
XLIBPATH            =  -L/usr/X11R6/lib
XLIB                =  -lSM -lICE -lX11
XTLIB               = -lXt
XMLIB               =  -lXm 
XEXTRALIBS          = 
USE_X11             = 1

COMPATLIB           = -lcompat
CURSES              = -lncurses $(COMPATLIB)
USE_CURSES          = 1
MATHLIB             =  -lm
XDRLIB              =     -lz 
ICONVLIB            = -liconv
INTLLIB             = -lintl
SOCKLIB             = 

#ZLIB:
ZLIB                =    -lz 
ZLIBINCPATH         = 
ZLIBLIBPATH         = 

#DBM:
#DBMINCPATH          = @DBMINCPATH@
#DBMLIBPATH          = @DBMLIBPATH@
#DBMLIB              = @DBMLIB@
DBMIEXTRALIB        = -lcompat 

#readline
READLINEINCPATH     = 
READLINELIBPATH     = 
READLINELIB         =  -lreadline 
HISTORYLIB          =  -lhistory 

#PostgreSQL:
PQINCPATH           = 
PQLIBPATH           = 
PQLIB               =  -lpq 

#MySQL:
MYSQLINCPATH        = 
MYSQLLIBPATH        = 
MYSQLLIB            = 
MYSQLDLIB            = 

#SQLite:
SQLITEINCPATH        = 
SQLITELIBPATH        = 
SQLITELIB            = 

#FFMPEG
FFMPEGINCPATH        = 
FFMPEGLIBPATH        = 
FFMPEGLIB            = 

#ODBC:
ODBCINC             = 
ODBCLIB             = 

#Image formats:
PNGINC              = 
PNGLIB              =  -lpng  -lz  
USE_PNG             = 1

JPEGINCPATH         = 
JPEGLIBPATH         = 
JPEGLIB             =  -ljpeg 

TIFFINCPATH         = 
TIFFLIBPATH         = 
TIFFLIB             =  -ltiff 

#GD library:
#GDINC               = @GDINC@
#GDLIB               = @GDLIB@
#USE_GD              = @USE_GD@

#openGL files for NVIZ/r3.showdspf
OPENGLINC           =  -I/home/geni/usr/local/include
OPENGLWINC          =  -I/home/geni/usr/local/include
OPENGLLIB           =  /home/geni/usr/local/lib/libGL.so 
OPENGLULIB          =  /home/geni/usr/local/lib/libGLU.so 
OPENGLWM            =  /home/geni/usr/local/lib/libGLw.so 
#USE_GLWM            = @USE_GLWM@

#tcl/tk stuff
TCLINCDIR           =  -I/usr/local/include/tcl8.4 -I/usr/local/include/tk8.4
TKINCDIR            =  -I/usr/local/include/tcl8.4 -I/usr/local/include/tk8.4
TCLTKLIBPATH        = 
TCLTKLIBS           =  -ltk84   -ltcl84 

#FFTW:
FFTWINC             = 
FFTWLIB             =  -lfftw 

#LAPACK/BLAS stuff for gmath lib:
BLASLIB             = 
LAPACKLIB           = 

#GDAL/OGR
GDALLIBS            = /home/geni/usr/local/lib/libgdal.so
GDALCFLAGS          = -I/home/geni/usr/local/include
USE_GDAL            = 1
USE_OGR             = 1

#FreeType:
FTINC               =  -I/usr/local/include/freetype2
FTLIB               =  -lfreetype 

#PROJ.4:
PROJINC             =  $(GDALCFLAGS)
PROJLIB             =  -lproj 
NAD2BIN             = /usr/local/bin/nad2bin

#OPENDWG:
OPENDWGINCPATH      = 
OPENDWGLIBPATH      = 
OPENDWGLIB          = 
USE_OPENDWG         = 

#i18N
HAVE_NLS            = 1

#Large File Support (LFS)
USE_LARGEFILES     = 1

MINGW		= 

# Cross compilation
CROSS_COMPILING =  
