/*
 *	iopen -
 *
 *				Paul Haeberli - 1984
 *
 */
#include	<stdio.h>
#include	<stdlib.h>
#include        <ctype.h>
#include	<fcntl.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	"image.h"
#include	"local_proto.h"


#undef PARM
#ifdef __STDC__
#  define PARM(a) a
#else
#  define PARM(a) ()
#  define const
#endif


static u_short  getshort      PARM((FILE *));
static u_long   getlong       PARM((FILE *));
static void     putshort      PARM((FILE *, int));
static void     putlong       PARM((FILE *, u_long));


typedef unsigned char byte;


IMAGE *iopen(char *file, char *mode, u_int type, u_int dim,
             u_int xsize, u_int ysize, u_int zsize)
{
    return(imgopen(0, file, mode, type, dim, xsize, ysize, zsize));
}

IMAGE *fiopen(int f, char *mode, u_int type, u_int dim,
              u_int xsize, u_int ysize, u_int zsize)
{
    return(imgopen(f, 0, mode, type, dim, xsize, ysize, zsize));
}

IMAGE *imgopen(int f, char *file, char *mode, u_int type, u_int dim, 
               u_int xsize, u_int ysize, u_int zsize)
{
	register IMAGE 	*image;
	register rw;
	size_t tablesize;
	register int i, max;
	FILE *f1 = NULL;

	image = (IMAGE *)calloc(sizeof(char), sizeof(IMAGE));
	rw = mode[1] == '+';
	if(rw) {
	    i_errhdlr("iopen: read/write mode not supported\n");
		return NULL;
	}
	if (*mode=='w') {
		if (file) {
		    f = creat(file, 0666);
		f1 = fopen(file,"w");
		    if (rw && f>=0) {
			    close(f);
			    f = open(file, 2);
		    }
		}
		if (f < 0) {
		    i_errhdlr("iopen: can't open output file %s\n",file);
		    return NULL;
		}
		image->imagic = IMAGIC;
		image->type = type;
		image->xsize = xsize;
		image->ysize = 1;
		image->zsize = 1;
		if (dim>1)
		    image->ysize = ysize;
		if (dim>2)
		    image->zsize = zsize;
		if(image->zsize == 1) {
		    image->dim = 2;
		    if(image->ysize == 1)
			image->dim = 1;
		} else {
		    image->dim = 3;
		}
		image->min = 10000;
		image->max = 0;
		isetname(image,"no name"); 
		image->wastebytes = 0;
		image->dorev = 0;

  /* byte order independent */
  fwrite(image, sizeof(IMAGE), sizeof(char), f1);
  fseek(f1, 0L, 0);

  putshort(f1, image->imagic);
  putshort(f1, image->type);
  putshort(f1, image->dim);
  putshort(f1, image->xsize);
  putshort(f1, image->ysize);
  putshort(f1, image->zsize);
  putlong (f1, image->min);
  putlong (f1, image->max);
  putlong (f1, 0L);
  fwrite  ("no name", 8 * sizeof(char), sizeof(char), f1);

  if (ferror(f1)) { fclose(f1);  return NULL; }

  fclose(f1);


/*		if (write(f,image,sizeof(IMAGE)) != sizeof(IMAGE)) {
		    i_errhdlr("iopen: error on write of image header\n");
		    return NULL;
		}*/
	} else {
		if (file)
		    f = open(file, rw? 2: 0);
		if (f < 0)
		    return(NULL);
		if (read(f,image,sizeof(IMAGE)) != sizeof(IMAGE)) {
		    i_errhdlr("iopen: error on read of image header\n");
		    return NULL;
		}
		if( ((image->imagic>>8) | ((image->imagic&0xff)<<8)) 
							     == IMAGIC ) {
		    image->dorev = 1;
		    cvtimage(image);
		} else
		    image->dorev = 0;
		if (image->imagic != IMAGIC) {
			i_errhdlr("iopen: bad magic in image file %x\n",image->imagic);
		    return NULL;
		}
	}
	if (rw)
	    image->flags = _IORW;
	else if (*mode != 'r')
	    image->flags = _IOWRT;
	else
	    image->flags = _IOREAD;

	if (ISRLE(image->type)) {
	    tablesize = image->ysize*image->zsize*sizeof(long);
	    image->rowstart = (unsigned long *)malloc(tablesize);
	    image->rowsize = (long *)malloc(tablesize);

	    if( image->rowstart == 0 || image->rowsize == 0 ) {
		i_errhdlr("iopen: error on table alloc\n");
		return NULL;
	    }
	    image->rleend = 512L+2*tablesize;
	    if (*mode=='w') {
		max = image->ysize*image->zsize;
		for(i=0; i<max; i++) {
		    image->rowstart[i] = 0;
		    image->rowsize[i] = -1;
		}
	    } else {
		tablesize = image->ysize*image->zsize*sizeof(long);
		lseek(f, 512L, 0);
		if (read(f,image->rowstart,tablesize) != tablesize) {
		    i_errhdlr("iopen: error on read of rowstart\n");
		    return NULL;
		}
		if(image->dorev)
		    cvtlongs(image->rowstart,tablesize);
		if (read(f,image->rowsize,tablesize) != tablesize) {
		    i_errhdlr("iopen: error on read of rowsize\n");
		    return NULL;
		}
		if(image->dorev)
		    cvtlongs(image->rowsize,tablesize);
	    }
	}
	image->cnt = 0;
	image->ptr = 0;
	image->base = 0;
	if( (image->tmpbuf = ibufalloc(image)) == 0 ) {	
	    i_errhdlr("iopen: error on tmpbuf alloc %d\n",image->xsize);
	    return NULL;
	}
	image->x = image->y = image->z = 0;
	image->file = f;
	image->offset = 512L;			/* set up for img_optseek */
	lseek(image->file, 512L, 0);

	return(image);
}

unsigned short *ibufalloc(IMAGE *image)
{
    return (unsigned short *)malloc(IBUFSIZE(image->xsize));
}

unsigned long reverse(u_long lwrd) 
{
    return ((lwrd>>24) 		| 
	   (lwrd>>8 & 0xff00) 	| 
	   (lwrd<<8 & 0xff0000) | 
	   (lwrd<<24) 		);
}

void cvtshorts(u_short buffer[], long n)
{
    register short i;
    register long nshorts = n>>1;
    register unsigned short swrd;

    for(i=0; i<nshorts; i++) {
	swrd = *buffer;
	*buffer++ = (swrd>>8) | (swrd<<8);
    }
}

void cvtlongs(long buffer[], long n)
{
    register short i;
    register long nlongs = n>>2;
    register unsigned long lwrd;

    for(i=0; i<nlongs; i++) {
	lwrd = buffer[i];
	buffer[i] =     ((lwrd>>24) 		| 
	   		(lwrd>>8 & 0xff00) 	| 
	   		(lwrd<<8 & 0xff0000) 	| 
	   		(lwrd<<24) 		);
    }
}

void cvtimage(long buffer[])
{
    cvtshorts(buffer,12);
    cvtlongs(buffer+3,12);
    cvtlongs(buffer+26,4);
}

static void (*i_errfunc)() = 0;

/*	error handler for the image library.  If the iseterror() routine
	has been called, sprintf's the args into a string and calls the
	error function.  Otherwise calls fprintf with the args and then
	exit.  This allows 'old' programs to assume that no errors
	ever need be worried about, while programs that know how and
	want to can handle the errors themselves.  Olson, 11/88
*/
void i_errhdlr(char *fmt, ...)
{
	va_list ap;
	char buffer[2000];

	va_start(ap, fmt);
	vsprintf(buffer, fmt, ap);
	va_end(ap);

	if (i_errfunc)
	    (*i_errfunc)(buffer);
	else
	    fprintf(stderr, "ERROR: %s", buffer);
}

/* this function sets the error handler for i_errhdlr */
void i_seterror(void (*func)())
{
	i_errfunc = func;
}

/* byte order independent read/write of shorts and longs. */

static u_short getshort(FILE *inf)
{
  byte buf[2];
  fread(buf, sizeof(short), sizeof(char), inf);
  return (buf[0]<<8)+(buf[1]<<0);
}


static u_long getlong(FILE *inf)
{
  byte buf[4];
  fread(buf, 4 * sizeof(char), sizeof(char), inf);
  return (((u_long) buf[0])<<24) + (((u_long) buf[1])<<16)
       + (((u_long) buf[2])<<8) + buf[3];
}


static void putshort(FILE *outf, int val)
{
  byte buf[2];
  buf[0] = (val>>8);
  buf[1] = (val>>0);
  fwrite(buf, sizeof(short), sizeof(char), outf);
}


static void putlong(FILE *outf, u_long val)
{
  byte buf[4];
  buf[0] = (val>>24);
  buf[1] = (val>>16);
  buf[2] = (val>>8);
  buf[3] = (val>>0);
  fwrite(buf, 4 * sizeof(char), sizeof(char), outf);
}
